package com.glxp.mipsdl.client;

import com.glxp.mipsdl.req.base.*;
import com.glxp.mipsdl.req.post.PostFileThrOrderRequest;
import com.glxp.mipsdl.req.post.PostThrCorpRequest;
import com.glxp.mipsdl.req.post.PostThrInvProductsRequest;
import com.glxp.mipsdl.req.post.PostThrProductsRequest;
import com.glxp.mipsdl.res.BaseResponse;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * http请求接口
 */
public interface BaseHttpClient {

    /**
     * 获取往来单位列表
     *
     * @param testUnitRequest
     * @return
     */
    BaseResponse getUnit(UdiwmsUnitRequest udiwmsUnitRequest);

    /**
     * 获取耗材字典列表
     *
     * @param udiwmsProductRequest
     * @return
     */
    BaseResponse getProducts(UdiwmsProductRequest udiwmsProductRequest);

    /**
     * 获取单据类型列表
     *
     * @param udiwmsOrderRequest
     * @return
     */
    BaseResponse getOrderTypes(UdiwmsBusTypeRequest udiwmsBusTypeRequest);

    /**
     * 获取单据列表
     *
     * @param udiwmsOrderQueryRequest
     * @return
     */
    BaseResponse getOrders(UdiwmsOrderQueryRequest udiwmsOrderQueryRequest);

    /**
     * 提交单据接口
     *
     * @param udiwmsOrderRequest
     * @return
     */
    BaseResponse submitOrders(UdiwmsOrderRequest udiwmsOrderRequest);

    /**
     * 获取库存产品列表
     *
     * @param udiwmsInvProductsRequest
     * @return
     */
    BaseResponse getInvProducts(UdiwmsInvProductsRequest udiwmsInvProductsRequest);

    /**
     * 获取仓库列表
     *
     * @param udiwmsWarehouseRequest
     * @return
     */
    BaseResponse getWarehouse(UdiwmsWarehouseRequest udiwmsWarehouseRequest);


    /**
     * 测试接口连通性
     *
     * @return
     */
    BaseResponse testConnect();

    //TODO 添加 患者信息查询接口，并补全接口文档

    /**
     * 第三方系统查询UDI管理系统产品库存信息
     *
     * @param queryUdiInvProductsRequest
     * @return
     */
    BaseResponse queryUdiInvProducts(UdiwmsQueryUdiInvProductsRequest queryUdiInvProductsRequest);


    //添加产品，往来单位等字典信息到第三方系统

    /**
     * 新增产品信息
     *
     * @return
     */
    BaseResponse submitProducts(UdiwmsAddProductsRequest udiwmsAddProductsRequest);

    /**
     * 新增往来单位信息
     *
     * @return
     */
    BaseResponse submitUnit(UdiwmsAddUnitRequest udiwmsAddUnitRequest);

    /**
     * 提交预入库单据
     *
     * @param udiwmsPreInOrderRequest
     * @return
     */
    BaseResponse submitPreInOrders(@RequestBody UdiwmsPreInOrderRequest udiwmsPreInOrderRequest);


    //文件导入数据到UDI管理系统接口

    /**
     * 文件方式新增产品信息到UDI管理系统
     *
     * @param postThrProductsRequest
     * @return
     */
    BaseResponse postProducts(PostThrProductsRequest postThrProductsRequest);

    /**
     * 新增往来单位信息到UDI管理系统
     *
     * @param postThrCorpRequest
     * @return
     */
    BaseResponse postCorps(PostThrCorpRequest postThrCorpRequest);

    /**
     * 文件方式添加单据到UDI管理系统
     *
     * @param postFileThrOrderRequest
     * @return
     */
    BaseResponse postOrderDetail(PostFileThrOrderRequest postFileThrOrderRequest);

    /**
     * 文件方式添加产品库存到UDI管理系统
     *
     * @param postThrInvProductsRequest
     * @return
     */
    BaseResponse postInvProducts(PostThrInvProductsRequest postThrInvProductsRequest);
}
