package com.glxp.mipsdl.client.phxyy;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.glxp.mipsdl.client.BaseHttpClient;
import com.glxp.mipsdl.config.ThirdSysConfig;
import com.glxp.mipsdl.constant.ConstantType;
import com.glxp.mipsdl.constant.Constants;
import com.glxp.mipsdl.dao.auth.AuthWarehouseDao;
import com.glxp.mipsdl.dao.basic.BasicBussinessTypeDao;
import com.glxp.mipsdl.dao.basic.BasicCorpDao;
import com.glxp.mipsdl.dao.inout.IoCodeDao;
import com.glxp.mipsdl.dao.inout.IoOrderDao;
import com.glxp.mipsdl.dao.inout.IoOrderDetailBizDao;
import com.glxp.mipsdl.dao.thrsys.ThrBustypeOriginDao;
import com.glxp.mipsdl.dao.thrsys.ThrInvWarehouseDao;
import com.glxp.mipsdl.dao.thrsys.ThrSystemBusApiDao;
import com.glxp.mipsdl.entity.auth.AuthWarehouseEntity;
import com.glxp.mipsdl.entity.basic.BasicBussinessTypeEntity;
import com.glxp.mipsdl.entity.inout.IoCodeEntity;
import com.glxp.mipsdl.entity.inout.IoOrderDetailBizEntity;
import com.glxp.mipsdl.entity.inout.IoOrderEntity;
import com.glxp.mipsdl.entity.thrsys.ThrCorpEntity;
import com.glxp.mipsdl.entity.thrsys.ThrInvWarehouseEntity;
import com.glxp.mipsdl.entity.thrsys.ThrSystemBusApiEntity;
import com.glxp.mipsdl.http.HttpClient;
import com.glxp.mipsdl.req.base.*;
import com.glxp.mipsdl.req.phxyy.PhBusTypeRequest;
import com.glxp.mipsdl.req.phxyy.PhOrderRequest;
import com.glxp.mipsdl.req.phxyy.PhProductRequest;
import com.glxp.mipsdl.req.phxyy.PhUnitRequest;
import com.glxp.mipsdl.req.post.PostFileThrOrderRequest;
import com.glxp.mipsdl.req.post.PostThrCorpRequest;
import com.glxp.mipsdl.req.post.PostThrInvProductsRequest;
import com.glxp.mipsdl.req.post.PostThrProductsRequest;
import com.glxp.mipsdl.res.BaseResponse;
import com.glxp.mipsdl.service.auth.AuthWarehouseService;
import com.glxp.mipsdl.service.thrsys.ThrCorpService;
import com.glxp.mipsdl.service.thrsys.ThrSystemDetailService;
import com.glxp.mipsdl.util.DateUtil;
import com.glxp.mipsdl.util.DictUtl;
import com.glxp.mipsdl.util.ResultVOUtils;
import com.glxp.mipsdl.util.SubmitOrderUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 平和县医院对接物资系统Client
 */
@Slf4j
@Service(value = "100005HttpClient") //bean注册名称，使用编码 + HttpClient，文件接口加 FileClient
public class PhxyyClient implements BaseHttpClient {

    @Resource
    private ThrSystemDetailService thrSystemDetailService;
    @Resource
    private HttpClient httpClient;
    @Resource
    private ThrBustypeOriginDao thrBustypeOriginDao;
    @Resource
    private IoOrderDao orderDao;
    @Resource
    private ThrSystemBusApiDao thrSystemBusApiDao;
    @Resource
    private BasicBussinessTypeDao basicBussinessTypeDao;
    @Resource
    private BasicCorpDao basicCorpDao;
    @Resource
    private IoCodeDao codeDao;
    @Resource
    private AuthWarehouseDao authWarehouseDao;
    @Resource
    private ThrInvWarehouseDao thrInvWarehouseDao;
    @Resource
    private AuthWarehouseService authWarehouseService;
    @Resource
    private ThirdSysConfig thirdSysConfig;
    @Resource
    private ThrCorpService thrCorpService;
    @Resource
    private IoOrderDetailBizDao orderDetailBizDao;
    @Resource
    private SubmitOrderUtil submitOrderUtil;

    @Override
    public BaseResponse getUnit(UdiwmsUnitRequest udiwmsUnitRequest) {
        PhUnitRequest phUnitRequest = new PhUnitRequest();
        BeanUtil.copyProperties(udiwmsUnitRequest, phUnitRequest);
        String response = httpClient.postJson(thrSystemDetailService.getUrl(Constants.URL_NAME_CORP_URL), phUnitRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse getProducts(UdiwmsProductRequest udiwmsProductRequest) {
        PhProductRequest phProductRequest = new PhProductRequest();
        BeanUtil.copyProperties(udiwmsProductRequest, phProductRequest);
        String response = httpClient.postJson(thrSystemDetailService.getUrl(Constants.URL_NAME_PI_QUERY_URL), phProductRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse getOrderTypes(UdiwmsBusTypeRequest udiwmsBusTypeRequest) {
        PhBusTypeRequest phBusTypeRequest = new PhBusTypeRequest();
        BeanUtil.copyProperties(udiwmsBusTypeRequest, phBusTypeRequest);
        String response = httpClient.postJson(thrSystemDetailService.getUrl(Constants.URL_NAME_BUS_TYPE_QUERY_URL), phBusTypeRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse getOrders(UdiwmsOrderQueryRequest udiwmsOrderQueryRequest) {
        if (StrUtil.isBlank(udiwmsOrderQueryRequest.getBillType())) {
            return ResultVOUtils.error(500, "单据类型不能为空");
        }
        //查询绑定的第三方单据类型
        String thirdAction = thrBustypeOriginDao.findThirdActionByBillType(udiwmsOrderQueryRequest.getBillType(), DictUtl.CURRENT_HOSP_CODE);
        udiwmsOrderQueryRequest.setBillType(thirdAction);
        String response = httpClient.postJson(thrSystemDetailService.getUrl(Constants.URL_NAME_ORDER_QUERY_URL), udiwmsOrderQueryRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse submitOrders(UdiwmsOrderRequest udiwmsOrderRequest) {
        if (StrUtil.isBlank(udiwmsOrderRequest.getBillNo())) {
            return ResultVOUtils.error(500, "单据号不能为空");
        }

        PhOrderRequest phOrderRequest = new PhOrderRequest();
        BeanUtil.copyProperties(udiwmsOrderRequest, phOrderRequest);

        //查询单据和详情信息，组装第三方系统接口参数
        IoOrderEntity orderEntity = orderDao.selectOne(new QueryWrapper<IoOrderEntity>().eq("billNo", udiwmsOrderRequest.getBillNo()));

        phOrderRequest.setBillDate(DateUtil.formatDateTime(orderEntity.getCreateTime()));
        //查询对应的第三方单据类型
        ThrSystemBusApiEntity thrSystemBusApiEntity = thrSystemBusApiDao.selectOne(new QueryWrapper<ThrSystemBusApiEntity>().eq("code", udiwmsOrderRequest.getBillType()));
        if (StrUtil.isBlank(thrSystemBusApiEntity.getUrl())) {
            return ResultVOUtils.error(500, "接口未配置，无法提交！");
        }

        phOrderRequest.setBillType(thrSystemBusApiEntity.getThirdBuyCode());

        //查询单据类型，设置往来单位信息
        BasicBussinessTypeEntity basicBussinessTypeEntity = basicBussinessTypeDao.selectByAction(udiwmsOrderRequest.getBillType());
        if (basicBussinessTypeEntity.getCorpType().equals(Constants.CORP_HOSPTIAL)) {
            //医院客户
            phOrderRequest.setCorpId(orderEntity.getFromCorp());
            //查询客户名称
            String corpName = basicCorpDao.selectNameByErpId(orderEntity.getFromCorp());
            phOrderRequest.setCorpName(corpName);
        } else if (basicBussinessTypeEntity.getCorpType().equals(Constants.CORP_INTTERNAL)) {
            //内部科室
            AuthWarehouseEntity warehouseEntity = authWarehouseDao.selectOne(new QueryWrapper<AuthWarehouseEntity>().eq("code", udiwmsOrderRequest.getCorpId()));
            try {
                //根据第三方系统标识，获取对应字段的值
                ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectOne(new QueryWrapper<ThrInvWarehouseEntity>()
                        .eq("code", authWarehouseService.getThirdInvCode(warehouseEntity))
                        .eq("thirdSys", thirdSysConfig.getThirdId()));

                phOrderRequest.setCorpId(thrInvWarehouseEntity.getCode());
                phOrderRequest.setCorpName(thrInvWarehouseEntity.getName());
            } catch (Exception e) {
                log.error("查询单据对应往来单位的第三方仓库信息异常", e);
            }
        } else if (basicBussinessTypeEntity.getCorpType().equals(Constants.CORP_SP)) {
            //供应商
            ThrCorpEntity thrCorp = thrCorpService.getThrCorp(udiwmsOrderRequest.getCorpId());
            if (null != thrCorp) {
                phOrderRequest.setCorpId(thrCorp.getUnitId());
                phOrderRequest.setCorpName(thrCorp.getName());
            } else {
                log.error("查询单据对应的第三方往来单位信息异常");
            }
        }

        //查询仓库信息
        AuthWarehouseEntity warehouseEntity = authWarehouseDao.selectOne(new QueryWrapper<AuthWarehouseEntity>().eq("code", orderEntity.getInvCode()));
        try {
            ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectOne(new QueryWrapper<ThrInvWarehouseEntity>()
                    .eq("code", authWarehouseService.getThirdInvCode(warehouseEntity))
                    .eq("thirdSys", thirdSysConfig.getThirdId()));

            phOrderRequest.setLocInvCode(thrInvWarehouseEntity.getCode());
            phOrderRequest.setLocInvName(thrInvWarehouseEntity.getName());

            phOrderRequest.setLocStorageCode(thrInvWarehouseEntity.getCode());
            phOrderRequest.setLocStorageName(thrInvWarehouseEntity.getName());
        } catch (Exception e) {
            log.info("查询第三方系统仓库信息异常", e);
        }

        if (StrUtil.nullToEmpty(thrSystemBusApiEntity.getUrl()).equals("useOut")) {
            phOrderRequest.setCorpId(phOrderRequest.getLocStorageCode());
            phOrderRequest.setCorpName(phOrderRequest.getLocStorageName());

            ThrInvWarehouseEntity thrInvWarehouseEntity = thrInvWarehouseDao.selectOne(new QueryWrapper<ThrInvWarehouseEntity>().eq("code", "29").eq("thirdSysFk", thirdSysConfig.getThirdId()));
            phOrderRequest.setLocStorageCode("29");
            phOrderRequest.setLocStorageName(thrInvWarehouseEntity.getName());
        }

        phOrderRequest.setBillFlag("1");
        //设置单据出入库类型
        if (basicBussinessTypeEntity.getMainAction().equals(ConstantType.TYPE_PUT)) {
            phOrderRequest.setInoutType(1); //入库
        } else if (basicBussinessTypeEntity.getMainAction().equals(ConstantType.TYPE_OUT)) {
            phOrderRequest.setInoutType(2); //出库
        }

        phOrderRequest.setRemark("udi管理系统提交单据");

        //组装单据明细数据
        List<IoCodeEntity> ioCodeEntities = codeDao.selectList(new QueryWrapper<IoCodeEntity>()
                .eq("orderId", orderEntity.getBillNo())
                .ne("action", "StockCheck")
        );

        List<UdiwmsOrderDetailRequest> items = new ArrayList<>();
        //查询业务单据详情
        List<IoOrderDetailBizEntity> detailBizEntities = orderDetailBizDao.selectList(new QueryWrapper<IoOrderDetailBizEntity>().eq("orderIdFk", orderEntity.getBillNo()));


        setOrderDetailList(ioCodeEntities, items, detailBizEntities);

        Collections.reverse(items);
        phOrderRequest.setItem(items);

        return submitOrderUtil.submitOrder(orderEntity, phOrderRequest);
    }

    /**
     * 封装单据详情参数
     *
     * @param ioCodeEntities      单据码明细
     * @param orderDetailRequests 单据详情参数
     * @param detailBizEntities   单据业务详情
     */
    private void setOrderDetailList(List<IoCodeEntity> ioCodeEntities, List<UdiwmsOrderDetailRequest> orderDetailRequests, List<IoOrderDetailBizEntity> detailBizEntities) {
        for (IoOrderDetailBizEntity detailBizEntity : detailBizEntities) {
            UdiwmsOrderDetailRequest item = submitOrderUtil.buildProductInfo(detailBizEntity);

            //封装条码信息
            submitOrderUtil.setCodeList(ioCodeEntities, detailBizEntity, item);

            orderDetailRequests.add(item);
        }
    }

    @Override
    public BaseResponse getInvProducts(UdiwmsInvProductsRequest udiwmsInvProductsRequest) {
        String response = httpClient.postJson(thrSystemDetailService.getUrl(Constants.URL_NAME_INV_PI_URL), udiwmsInvProductsRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse getWarehouse(UdiwmsWarehouseRequest udiwmsWarehouseRequest) {
        String response = httpClient.postJson(thrSystemDetailService.getUrl(Constants.URL_NAME_WAREHOUSE_QUERY_URL), udiwmsWarehouseRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse postProducts(PostThrProductsRequest postThrProductsRequest) {
        postThrProductsRequest.setUploadType("文件导入");
        String response = httpClient.postUDIWMSJson(thirdSysConfig.getPostPiUrl(), postThrProductsRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse postCorps(PostThrCorpRequest postThrCorpRequest) {
        postThrCorpRequest.setUploadType("文件导入");
        String response = httpClient.postUDIWMSJson(thirdSysConfig.getPostCorpUrl(), postThrCorpRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse postOrderDetail(PostFileThrOrderRequest postFileThrOrderRequest) {
        postFileThrOrderRequest.setUploadType("文件导入");
        String response = httpClient.postUDIWMSJson(thirdSysConfig.getPostOrderUrl(), postFileThrOrderRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse postInvProducts(PostThrInvProductsRequest postThrInvProductsRequest) {
        String response = httpClient.postUDIWMSJson(thirdSysConfig.getPostInvPiUrl(), postThrInvProductsRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse testConnect() {
        UdiwmsUnitRequest udiwmsUnitRequest = new UdiwmsUnitRequest();
        udiwmsUnitRequest.setPage(1);
        udiwmsUnitRequest.setLimit(2);
        BaseResponse baseResponse = getUnit(udiwmsUnitRequest);
        if (baseResponse.getCode() == 20000) {
            return ResultVOUtils.success("连接成功！");
        }
        return ResultVOUtils.error(500, "连接失败！");
    }

    @Override
    public BaseResponse queryUdiInvProducts(UdiwmsQueryUdiInvProductsRequest queryUdiInvProductsRequest) {
        String response = httpClient.postUDIWMSJson(thirdSysConfig.getQueryInvUrl(), queryUdiInvProductsRequest);
        BaseResponse baseResponse = JSONObject.parseObject(response, BaseResponse.class);
        return baseResponse;
    }

    @Override
    public BaseResponse submitProducts(UdiwmsAddProductsRequest udiwmsAddProductsRequest) {
        return null;
    }

    @Override
    public BaseResponse submitUnit(UdiwmsAddUnitRequest udiwmsAddUnitRequest) {
        return null;
    }

    @Override
    public BaseResponse submitPreInOrders(UdiwmsPreInOrderRequest udiwmsPreInOrderRequest) {
        return null;
    }
}
